
(function(app) {
    'use strict';

    app.factory("MonthPicker", ["$rootScope", "$ionicPopup", function($rootScope, $ionicPopup) {
        var scope;

        return {
            /**
             * Call for initialization
             * @param options
             */
            init: function(options) {
                scope = $rootScope.$new();
           
                scope.restData = function(options) {
                    //
                    // Options and defaults
                    //
                    scope.minMonthIndex = options.minMonthIndex || 0;
                    scope.minYear = options.minYear || new Date().getFullYear();

                    scope.maxMonthIndex = options.maxMonthIndex === undefined ? new Date().getMonth() : options.maxMonthIndex;

                    scope.maxYear = options.maxYear || new Date().getFullYear();

                    scope.monthLabels = options.monthLabels || ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
                    scope.title = options.title || "Select month";
                    scope.cancelText = options.cancelText || "Cancel";
                    scope.cancelClass = options.cancelClass || 'button-cancle-mp';

                    scope.inputYear = options.inputYear;
                    scope.inputMonth = options.inputMonth;

                    scope.selection.year = scope.inputYear;
                    scope.selection.month = scope.inputMonth;
                }
              
                //
                // What user selects
                //
                scope.selection = {
                    year : options.startingYear || scope.maxYear,
                    month : null // no month selected
                };

                /**
                 * Arrow buttons
                 * @param index
                 */
                scope.changeYear = function(index) {
                    scope.selection.year += index;
                };
                /**
                 * verify month index is in required interval
                 * @param index
                 * @returns {boolean}
                 */
                scope.isValidMonth = function(index) {
                    var invalid = (index < scope.minMonthIndex && scope.selection.year == scope.minYear) ||
                        ( index > scope.maxMonthIndex && scope.selection.year == scope.maxYear) ||
                        scope.selection.year < scope.minYear ||
                        scope.selection.year > scope.maxYear;
                    return ! invalid;
                }
                /**
                 * verify month index is current seleted month
                 * @param index
                 * @returns {boolean}
                 */
                scope.isSelMonth = function(index) {
                    var selected = index == scope.inputMonth && scope.selection.year == scope.inputYear;
                    return selected;
                }

                scope.getMonthClass = function(index) {
                    if(!scope.isValidMonth(index)) {
                        return "dark";
                    } else if(scope.isSelMonth(index)) {
                        return "blue"
                    } else {
                        return "";
                    }
                }

                scope.restData(options);
            },

            /**
             * close the month picker
             * @param
             */
            close: function() {
                if(scope && scope.popup)
                    scope.popup.close();
            },
            
            /**
             * Show the month picker
             * @param callback
             */
            show: function(options, callback) {

                scope.restData(options);
                
                /**
                 * Tap on month
                 * @param index
                 */
                scope.selectMonth = function(index) {
                    if (scope.isValidMonth(index)) {
                        scope.selection.month = index;
                        scope.popup.close();
                    }
                };
                //
                // Actual popup
                //
                scope.popup = $ionicPopup.show({
                    cssClass: 'my-custom-popup',
                    templateUrl: "monthpicker.html",                    
                    title: scope.title,
                    scope: scope,
                    buttons: [
                        {
                            text: scope.cancelText,
                            type: scope.cancelClass,
                            onTap: function(e){
                              return true;
                            }
                        }
                    ]
                });

                scope.popup.then(function(res) {
                  if(res){
                    return null;
                  }                  
                    //return clone obj not internal private obj
                    callback({
                        year : scope.selection.year,
                        month : scope.selection.month
                    });
                });
            }
        };
    }]);

}(angular.module("ionic-monthpicker")));
